@@ -28,6 +28,9 @@ type PendingQueryResponses = Arc<DashMap<OperationId, Vec<(MessageId, QueryRespo
 use uluru::LRUCache;
 type QueryResponseSender = Sender<QueryResponse>;
 
+type CmdResponse = (std::net::SocketAddr, Option<CmdError>);
+type PendingCmdAcks = Arc<DashMap<MessageId, Sender<CmdResponse>>>;
+
 #[derive(Debug)]
 pub struct QueryResult {
     pub response: QueryResponse,
@@ -43,7 +46,10 @@ pub(super) struct Session {
     // Channels for sending responses to upper layers
     pending_queries: PendingQueryResponses,
     // Channels for sending errors to upper layer
+    #[allow(dead_code)]
     incoming_err_sender: Arc<Sender<CmdError>>,
+    // Channels for sending CmdAck to upper layers
+    pending_cmds: PendingCmdAcks,
     /// All elders we know about from AE messages
     network: Arc<NetworkPrefixMap>,
     /// AE redirect cache
