@@ -1,4 +1,4 @@
-// Copyright 2021 MaidSafe.net limited.
+// Copyright 2022 MaidSafe.net limited.
 //
 // This SAFE Network Software is licensed to you under The General Public License (GPL), version 3.
 // Unless required by applicable law or agreed to in writing, the SAFE Network Software distributed
@@ -86,9 +86,15 @@ pub enum Error {
     /// There were Error(s) while batching for Sled operations.
     #[error("Errors found when batching for Sled")]
     SledBatching,
+    /// Invalid filename
+    #[error("Invalid chunk filename")]
+    InvalidFilename,
     /// NetworkData error.
     #[error("Network data error:: {0}")]
     NetworkData(#[from] crate::types::Error),
+    /// No filename found
+    #[error("Path contains no file name")]
+    NoFilename,
 }
 
 /// Convert db error to messaging error message for sending over the network.
