@@ -9,7 +9,7 @@
 use crate::messaging::system::{KeyedSig, MembershipState, SectionAuth};
 use crate::node::{
     api::command::Command,
-    core::{Core, Proposal},
+    core::{relocation::ChurnId, Core, Proposal},
     dkg::SectionAuthUtils,
     network_knowledge::{NodeState, SectionAuthorityProvider},
     Event, Result, MIN_ADULT_AGE,
@@ -73,8 +73,10 @@ impl Core {
                 // having to send this `NodeApproval`.
                 commands.extend(self.send_node_approval(old_info.clone()).await);
 
-                let peer = new_info.peer().clone();
-                commands.extend(self.relocate_rejoining_peer(peer, new_age).await?);
+                commands.extend(
+                    self.relocate_rejoining_peer(old_info.value, new_age)
+                        .await?,
+                );
 
                 return Ok(commands);
             }
@@ -109,8 +111,10 @@ impl Core {
             *self.joins_allowed.write().await = false;
         }
 
+        let churn_id = ChurnId(new_info.sig.signature.to_bytes().to_vec());
+        let excluded_from_relocation = vec![new_info.name()].into_iter().collect();
         commands.extend(
-            self.relocate_peers(&new_info.name(), &new_info.sig.signature)
+            self.relocate_peers(churn_id, excluded_from_relocation)
                 .await?,
         );
 
@@ -138,12 +142,14 @@ impl Core {
         let mut commands = vec![];
         let signature = sig.signature.clone();
 
+        let signed_node_state = SectionAuth {
+            value: node_state.clone(),
+            sig,
+        };
+
         if !self
             .network_knowledge
-            .update_member(SectionAuth {
-                value: node_state.clone(),
-                sig,
-            })
+            .update_member(signed_node_state.clone())
             .await
         {
             info!(
@@ -160,7 +166,18 @@ impl Core {
             node_state.addr()
         );
 
-        commands.extend(self.relocate_peers(&node_state.name(), &signature).await?);
+        // If this is an Offline agreement where the new node state is Relocated,
+        // we then need to send the Relocate msg to the peer attaching the signed NodeState
+        // containing the relocation details.
+        if node_state.is_relocated() {
+            commands.extend(
+                self.send_relocate(node_state.peer().clone(), signed_node_state)
+                    .await?,
+            );
+        }
+
+        let churn_id = ChurnId(signature.to_bytes().to_vec());
+        commands.extend(self.relocate_peers(churn_id, BTreeSet::default()).await?);
 
         let result = self.promote_and_demote_elders().await?;
         if result.is_empty() {
