@@ -19,13 +19,13 @@ mod update_section;
 use crate::dbs::Error as DbError;
 use crate::node::{
     api::command::Command,
-    core::{relocation::RelocateState, Core, DkgSessionInfo},
+    core::{Core, DkgSessionInfo},
     messages::{NodeMsgAuthorityUtils, WireMsgUtils},
     network_knowledge::NetworkKnowledge,
     Error, Event, MessageReceived, Result, MIN_LEVEL_WHEN_FULL,
 };
 use crate::peer::{Peer, UnnamedPeer};
-use crate::types::{log_markers::LogMarker, Keypair, PublicKey};
+use crate::types::{log_markers::LogMarker, PublicKey, Signature};
 use crate::{
     messaging::{
         data::{ServiceMsg, StorageLevel},
@@ -42,7 +42,7 @@ use crate::{
 
 use bls::PublicKey as BlsPublicKey;
 use bytes::Bytes;
-use rand::rngs::OsRng;
+use ed25519_dalek::Signer;
 use std::collections::BTreeSet;
 use xor_name::XorName;
 
@@ -312,25 +312,13 @@ impl Core {
                 )
                 .await
             }
-            SystemMsg::Relocate(ref details) => {
+            SystemMsg::Relocate(node_state) => {
                 trace!("Handling msg: Relocate from {}: {:?}", sender, msg_id);
-                if let NodeMsgAuthority::Section(section_signed) = msg_authority {
-                    Ok(self
-                        .handle_relocate(details.clone(), node_msg, section_signed)
-                        .await?
-                        .into_iter()
-                        .collect())
-                } else {
-                    Err(Error::InvalidSrcLocation)
-                }
-            }
-            SystemMsg::RelocatePromise(promise) => {
-                trace!(
-                    "Handling msg: RelocatePromise from {}: {:?}",
-                    sender,
-                    msg_id
-                );
-                self.handle_relocate_promise(promise, node_msg).await
+                Ok(self
+                    .handle_relocate(node_state)
+                    .await?
+                    .into_iter()
+                    .collect())
             }
             SystemMsg::StartConnectivityTest(name) => {
                 trace!(
@@ -346,9 +334,7 @@ impl Core {
             }
             SystemMsg::JoinAsRelocatedResponse(join_response) => {
                 trace!("Handling msg: JoinAsRelocatedResponse from {}", sender);
-                if let Some(RelocateState::InProgress(ref mut joining_as_relocated)) =
-                    *self.relocate_state.write().await
-                {
+                if let Some(ref mut joining_as_relocated) = *self.relocate_state.write().await {
                     if let Some(cmd) = joining_as_relocated
                         .handle_join_response(*join_response, sender.addr())
                         .await?
@@ -356,7 +342,10 @@ impl Core {
                         return Ok(vec![cmd]);
                     }
                 } else {
-                    error!("self.relocate_state is not in Progress");
+                    error!(
+                        "No relocation in progress upon receiving {:?}",
+                        join_response
+                    );
                 }
 
                 Ok(vec![])
@@ -412,7 +401,7 @@ impl Core {
             }
             SystemMsg::BackPressure(load_report) => {
                 trace!("Handling msg: BackPressure from {}: {:?}", sender, msg_id);
-                // #TODO: Factor in med/long term backpressure into general node liveness calculations
+                // TODO: Factor in med/long term backpressure into general node liveness calculations
                 self.comm.regulate(sender.addr(), load_report).await;
                 Ok(vec![])
             }
@@ -424,6 +413,7 @@ impl Core {
                         sig_share,
                         section_chain,
                         members,
+                        ..
                     } => {
                         let serialized_details = bincode::serialize(&node_state)?;
 
@@ -440,9 +430,8 @@ impl Core {
                                 info!("Relocation: Successfully aggregated ApprovalShares for joining the network");
                                 let mut commands = vec![];
 
-                                if let Some(RelocateState::InProgress(
-                                    ref mut joining_as_relocated,
-                                )) = *self.relocate_state.write().await
+                                if let Some(ref mut joining_as_relocated) =
+                                    *self.relocate_state.write().await
                                 {
                                     let new_node = joining_as_relocated.node.clone();
                                     let new_name = new_node.name();
@@ -586,7 +575,6 @@ impl Core {
             } => {
                 if self.is_not_elder().await {
                     trace!("Adult handling a Propose msg from {}: {:?}", sender, msg_id);
-                    // return Ok(vec![]);
                 }
 
                 trace!("Handling msg: Propose from {}: {:?}", sender, msg_id);
@@ -654,8 +642,6 @@ impl Core {
                 self.handle_dkg_retry(session_id, message_history, message, sender)
                     .await
             }
-            // The following type of messages are all handled by upper sn_node layer.
-            // TODO: In the future the sn-node layer won't be receiving Events
             SystemMsg::NodeCmd(NodeCmd::RecordStorageLevel { node_id, level, .. }) => {
                 let changed = self.set_storage_level(&node_id, level).await;
                 if changed && level.value() == MIN_LEVEL_WHEN_FULL {
@@ -738,12 +724,13 @@ impl Core {
                         query,
                         auth,
                         origin,
+                        correlation_id,
                     } => {
                         // There is no point in verifying a sig from a sender A or B here.
                         // Send back response to the sending elder
                         let sender_xorname = msg_authority.get_auth_xorname();
                         self.handle_data_query_at_adult(
-                            msg_id,
+                            correlation_id,
                             &query,
                             auth,
                             origin,
@@ -1020,16 +1007,15 @@ impl Core {
         }
     }
 
-    // TODO: Dedupe this w/ node
-    fn random_client_signature(client_msg: &ServiceMsg) -> Result<(MsgKind, Bytes)> {
-        let mut rng = OsRng;
-        let keypair = Keypair::new_ed25519(&mut rng);
+    // Currently using node's Ed key. May need to use bls key share for concensus purpose.
+    async fn ed_sign_client_message(&self, client_msg: &ServiceMsg) -> Result<(MsgKind, Bytes)> {
+        let keypair = self.node.read().await.keypair.clone();
         let payload = WireMsg::serialize_msg_payload(client_msg)?;
         let signature = keypair.sign(&payload);
 
         let msg = MsgKind::ServiceMsg(ServiceAuth {
-            public_key: keypair.public_key(),
-            signature,
+            public_key: PublicKey::Ed25519(keypair.public),
+            signature: Signature::Ed25519(signature),
         });
 
         Ok((msg, payload))
